import { Router } from 'vue-router'
import { ArticleEntry } from '../content/templates/article-list'
import { GalleryEntry } from '../content/templates/gallery-list'
import { TemplateType } from '../content/templates/templateType'
import { WarningModal } from './warnings'
import type { mount } from 'mount-vue-component'

/**
 * Defines the base options for all routes
 */
export type BaseRouteDefinition = {
  /**
   * The short-form title of the page (used for breadcrumbs/any other place that `fullTitle` isn't used)
   */
  title: string
  /**
   * The full title of the page (used for document title)
   */
  fullTitle: string
  /**
   * Stylesheets to load on the page
   */
  stylesheetUrls?: string[]
  /**
   * Script to load on the page
   */
  scriptUrl?: string
}

/**
 * Defines the shared options for a route
 */
export type SharedRouteDefinition = BaseRouteDefinition & {
  /**
   * Unique identifier for the page
   */
  id: string
  /**
   * What kind of page the route is
   */
  template: TemplateType
  /**
   * Modal text shown by certain templates that may feature content warnings;
   * setting this to `false` will force the modal off regardless of global site config;
   * setting this to `true` or leaving it `undefined`/`null` will use the global site config;
   * setting this to {@link WarningModal} will override the global site config with the defined modal definition
   */
  warning?: boolean | WarningModal
}

/**
 * Defines a content-based route
 */
export type ContentfulRouteDefintion = SharedRouteDefinition & {
  /**
   * Url to the page's content
   */
  content: string
}

/**
 * Defines a config-based route
 */
export type ConfigfulRouteDefinition = SharedRouteDefinition & {
  /**
   * Url to the page's config
   */
  config: string
}

/**
 * Defines the config for a route using the `markdown` {@link TemplateType}
 */
export type MarkdownDefinition = ContentfulRouteDefintion & {
  template: 'markdown'
}

/**
 * Defines the config for a route using the `article-list` {@link TemplateType}
 */
export type ArticleListDefinition = ConfigfulRouteDefinition & {
  template: 'article-list'
  /**
   * Config for the child view `{parent/path}/view` route
   */
  view: BaseRouteDefinition
}

/**
 * Defines the config for a route using the `gallery-list` {@link TemplateType}
 */
export type GalleryListDefinition = ConfigfulRouteDefinition & {
  template: 'gallery-list'
  /**
   * Config for the child view `{parent/path}/view` route
   */
  view: BaseRouteDefinition
}

/**
 * Defines all available route definitions
 */
export type RouteDefinition = (
  | MarkdownDefinition
  | ArticleListDefinition
  | GalleryListDefinition
)

/**
 * Defines the collection of routes the app uses
 */
export type RouteCollection = { [key: string]: RouteDefinition }

/**
 * Defines {@link Window} globals
 */
export interface RoutedWindow extends Window {
  /** function registered by the vite-ssg build which will mount Vue components in place of elements they are replacing (e.g. image embeds with PrimeVue Image) */
  mountComponent: typeof mount
  /** refers to a template's primary route config; may briefly refer to sub config until a view is fully resolved */
  routeConfig: RouteDefinition
  /** refers to a template's sub config in the case of child routes under a template (e.g. article-view under article-list) */
  routeSubConfig: any
  /** refers to content config for various view routes (e.g. {@link GalleryEntry}, {@link ArticleEntry}, etc.) */
  routeContentConfig: any
  /** vue-router instance */
  router?: Router
}
